package controller;

import helpers.ItemHelper;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import model.Customer;
import model.Item;
import model.Order;
import storage.CustomerStorage;
import storage.ItemStorage;
import storage.OrderStorage;

public class CheckoutController extends HttpServlet {
     /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    
    private CustomerStorage customerStorage;
    
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        
        String action = request.getParameter("submit");
        if (action == null || action.isEmpty()) {
            action = "none";
        }
        
        List<Item> items = this.getItems();
        Map<Integer, Item> itemMap = new HashMap<>();
        for (Item item : items) {
            itemMap.put(item.getId(), item);
        }
        
        HttpSession session = request.getSession();
        session.setAttribute("items", itemMap);
        
        Map<Integer, Integer> cart = (Map) session.getAttribute("cart");
        if (cart == null) {
            cart = new HashMap<>();
        }

        String url = null;
        switch(action) {
            case "Checkout":
                if (!this.isAllItemsInStock(items, cart)) {
                    url = "/WEB-INF/view/orderfailed.jsp";
                    break;
                }
                ServletContext s = this.getServletContext();
                this.customerStorage = new CustomerStorage(s.getRealPath("/WEB-INF/xml"));
                int customerId = this.saveCustomer(request);
                this.saveOrder(customerId, cart);
                this.updateItemStockCount(items, cart);
                cart = new HashMap<>();
                url = "/WEB-INF/view/ordersuccess.jsp";
                break;
            case "remove":
                String itemIdString = request.getParameter("item_id");                
                int itemId = ItemHelper.parseItemId(itemIdString, items);
                cart.remove(itemId);
            default:
                // invalid action
        }
        
        session.setAttribute("cart", cart);   

        response.setContentType("text/html;charset=UTF-8");
        if (url == null) {
            url = "/WEB-INF/view/checkout.jsp";
        }
        ServletContext context = this.getServletContext();
        RequestDispatcher requestDispatcher = context.getRequestDispatcher(url);
        requestDispatcher.forward(request, response);
    }
    
    private List<Item> getItems() {
        ServletContext s = this.getServletContext();
        ItemStorage itemStorage = new ItemStorage(s.getRealPath("/WEB-INF/xml"));

        List<Item> items = new ArrayList<>();
        items = itemStorage.read();
        
        return items;
    }
    
    public boolean isAllItemsInStock(List<Item> items, Map<Integer, Integer> cart) {
        for (Item item : items) {
            Integer orderAmount = cart.get(item.getId());
            if (orderAmount == null) {
                continue;
            }
            if (orderAmount > item.getStockCount()) {
                return false;
            }
        }
        return true;
    }
    
    public void updateItemStockCount(List<Item> items, Map<Integer, Integer> cart) {
        ServletContext s = this.getServletContext();
        ItemStorage itemStorage = new ItemStorage(s.getRealPath("/WEB-INF/xml"));
        for (Item item : items) {
            Integer orderAmount = cart.get(item.getId());
            if (orderAmount == null) {
                continue;
            }
            try {
                int newStockCount = item.getStockCount() - orderAmount;
                itemStorage.updateStockCount(item.getId(), newStockCount);
            } catch (SQLException e) {
                e.printStackTrace();
            }  
        }
    }
    
    public int saveCustomer(HttpServletRequest request) {
        Customer customer = new Customer();
        customer.setFirstName(request.getParameter("first_name"));
        customer.setLastName(request.getParameter("last_name"));
        customer.setStreet(request.getParameter("street"));
        customer.setCity(request.getParameter("city"));
        customer.setEmail(request.getParameter("email"));

        return this.customerStorage.create(customer);
    }
    
    public void saveOrder(int customerId, Map<Integer, Integer> cart) {
        Order order = new Order();
        order.setCustomerId(customerId);
        order.setItemIdQuanitityMap(cart);
        
        ServletContext s = this.getServletContext();
        OrderStorage orderStorage = new OrderStorage(s.getRealPath("/WEB-INF/xml"));
        orderStorage.create(order);
    }
    
    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
