package controller;

import helpers.ItemHelper;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import model.Item;
import storage.ItemStorage;

public class Controller extends HttpServlet {
   
    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        String action = request.getParameter("action");
        if (action == null || action.isEmpty()) {
            action = "none";
        }
                        
        String itemIdString = request.getParameter("item_id");
        String itemAmountString = request.getParameter("amount");
        String filteredCompany = request.getParameter("company");
        
        HttpSession session = request.getSession();
        
        List<Item> items = this.getItems();
        Map<Integer, Item> itemMap = new HashMap<>();
        for (Item item : items) {
            itemMap.put(item.getId(), item);
        }
        
        List<String> companies = new ArrayList<>();
        for (Item item : items) {
            if (!companies.contains(item.getCompany())) {
                companies.add(item.getCompany());
            }
        }
          
        Map<Integer, Integer> cart = (Map) session.getAttribute("cart");
        if (cart == null) {
            cart = new HashMap<>();
        }

        int itemId;
        int itemAmount;
        switch(action) {
            case "add":
                itemId = ItemHelper.parseItemId(itemIdString, items);
                itemAmount = Integer.parseInt(itemAmountString);
                Integer itemQuanitity = cart.get(itemId);
                if (itemQuanitity == null) {
                    cart.put(itemId, itemAmount);
                } else {
                    itemQuanitity += itemAmount;
                    cart.put(itemId, itemQuanitity);
                }
                break;
            case "filter":
                Iterator<Map.Entry<Integer, Item>> it = itemMap.entrySet().iterator(); 
                while (it.hasNext() ) {
                    Map.Entry<Integer, Item> entry = it.next();
                    String company = entry.getValue().getCompany();
                    if (!company.equals(filteredCompany)) {
                        it.remove();
                    }
                }
                break;
            default:
                // invalid action
        }
        
        session.setAttribute("items", itemMap);
        session.setAttribute("companies", companies);
        session.setAttribute("cart", cart);   

        response.setContentType("text/html;charset=UTF-8");
        String url = "/WEB-INF/view/index.jsp";        
        ServletContext context = this.getServletContext();
        RequestDispatcher requestDispatcher = context.getRequestDispatcher(url);
        requestDispatcher.forward(request, response);
    }
    
    private List<Item> getItems() {
        ServletContext s = this.getServletContext();
        ItemStorage itemStorage = new ItemStorage(s.getRealPath("/WEB-INF/xml"));

        List<Item> items = new ArrayList<>();
        items = itemStorage.read();
        
        return items;
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
