package storage;

import generated.Items.Row;
import java.io.File;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import model.Item;

public class ItemStorage {
    
    private final String xmlDirectory; 
    
    public ItemStorage(String xmlDirectory) {
        this.xmlDirectory = xmlDirectory;
    }
   
    private final String ITEMS_TABLE = "/table_items.xml";
    
    public List<Item> read() {
        List<Item> items = new ArrayList<>();
        try {
            JAXBContext context = JAXBContext.newInstance(generated.Items.class);
            Unmarshaller u = context.createUnmarshaller();

            generated.Items itemsResult = (generated.Items) u.unmarshal(new File(xmlDirectory + ITEMS_TABLE));

            for(Row row: itemsResult.getRow()) {
                Item item = new Item();
                item.setId(row.getId());
                item.setName(row.getName());
                item.setCompany(row.getCompany());
                item.setDescription(row.getDescription());
                item.setPicture(row.getPictureUrl());
                item.setPrice(row.getPrice());
                item.setStockCount(row.getStockCount());
                
                items.add(item);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return items;
    }

    public void updateStockCount(int itemId, int stockCount) throws SQLException {
        try {
            JAXBContext context = JAXBContext.newInstance(generated.Items.class);
            Unmarshaller u = context.createUnmarshaller();

            generated.Items itemsJaxb = (generated.Items) u.unmarshal(new File(xmlDirectory + ITEMS_TABLE));

            for (Row row : itemsJaxb.getRow()) {
                if (row.getId() == itemId) {
                    row.setStockCount(stockCount);
                    break;
                }
            }
            
            Marshaller m = context.createMarshaller();
            m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

            m.marshal(itemsJaxb, new File(xmlDirectory + ITEMS_TABLE));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
}
