package storage;

import generated.Orders.Row;
import generated.OrdersItems;
import java.io.File;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.Random;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import model.Order;

public class OrderStorage {
    
    private final String ORDERS_TABLE = "/table_orders.xml";

    private final String ORDERS_ITEMS_TABLE = "/table_orders_items.xml";
    
    private final String xmlDirectory; 
    
    public OrderStorage(String xmlDirectory) {
        this.xmlDirectory = xmlDirectory;
    }
    
    public void create(Order order) {
        Random random = new Random();
        int orderId = random.nextInt(100000);
        
        try {
            JAXBContext context = JAXBContext.newInstance(generated.Orders.class);
            Unmarshaller u = context.createUnmarshaller();

            generated.Orders ordersJaxb = (generated.Orders) u.unmarshal(new File(xmlDirectory + ORDERS_TABLE));

            Row row = new Row();
            row.setId(orderId);
            row.setCustomerId(order.getCustomerId());
            row.setDate(this.getCurrentDate());
            
            ordersJaxb.getRow().add(row);
            
            Marshaller m = context.createMarshaller();
            m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

            m.marshal(ordersJaxb, new File(xmlDirectory + ORDERS_TABLE));
        } catch (Exception e) {
            return;
        }
        
        try {
            JAXBContext context = JAXBContext.newInstance(generated.OrdersItems.class);
            Unmarshaller u = context.createUnmarshaller();

            generated.OrdersItems ordersJaxb = (generated.OrdersItems) u.unmarshal(new File(xmlDirectory + ORDERS_ITEMS_TABLE));

            for (Map.Entry<Integer, Integer> entry : order.getItemIdQuanitityMap().entrySet()) {
                OrdersItems.Row row = new OrdersItems.Row();
                row.setOrderId(orderId);
                row.setItemId(entry.getKey());
                row.setQuanitity(entry.getValue());

                ordersJaxb.getRow().add(row);
            }
            
            Marshaller m = context.createMarshaller();
            m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

            m.marshal(ordersJaxb, new File(xmlDirectory + ORDERS_ITEMS_TABLE));
        } catch (Exception e) {
            
        }
    }
    
    private XMLGregorianCalendar getCurrentDate() {
        Date today = new Date();
        GregorianCalendar c = new GregorianCalendar();
        c.setTime(today);
        try {
            return DatatypeFactory.newInstance().newXMLGregorianCalendar(c);
        } catch (Exception e) {
            
        }
        return null;
    }
    
}
